#include "form.h"
#include "ui_form.h"
#include <QSizePolicy>
#include <QDebug>

Form::Form(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::Form)
{
    ui->setupUi(this);
    updateWidgetLabel(ui->Widget1);
    updateWidgetLabel(ui->Widget2);
    readWidgetProps(ui->Widget1);
}

Form::~Form()
{
    delete ui;
}

void Form::readWidgetProps(QWidget *widget)
{
    qDebug() << "read widget:" << widget->objectName();
    whileLoadingProps = true;
    ui->edtMinHeight->setValue(widget->minimumHeight());
    ui->edtMaxHeight->setValue(widget->maximumHeight());
    ui->edtStretchFactor->setValue(widget->sizePolicy().verticalStretch());
    ui->edtSizePolicy->setCurrentText(policyName(widget->sizePolicy().verticalPolicy()));
    whileLoadingProps = false;
}

void Form::saveWidgetProps(QWidget *widget)
{
    if (whileLoadingProps)
        return;

    qDebug() << "save widget" << widget->objectName();
    widget->setMinimumHeight(ui->edtMinHeight->value());
    widget->setMaximumHeight(ui->edtMaxHeight->value());
    QSizePolicy policy = QSizePolicy(widget->sizePolicy());
    policy.setVerticalStretch(ui->edtStretchFactor->value());
    policy.setVerticalPolicy(policyByName(ui->edtSizePolicy->currentText()));
    widget->setSizePolicy(policy);
    updateWidgetLabel(widget);
}

void Form::updateWidgetLabel(QWidget *widget)
{
    QString text = policyName(widget->sizePolicy().verticalPolicy());
    text.append(", Stretch: ");
    text.append(QString::number(widget->sizePolicy().verticalStretch()));
    text.append(", Min: ");
    text.append(QString::number(widget->minimumHeight()));
    text.append(", Max: ");
    text.append(QString::number(widget->maximumHeight()));
    QLabel* label2 = widget->findChildren<QLabel*>().first();
    label2->setText(text);
}

QString Form::policyName(QSizePolicy::Policy policy)
{
    switch (policy) {
    case QSizePolicy::Preferred:
        return "Preferred";
    case QSizePolicy::Expanding:
        return "Expanding";
    case QSizePolicy::Minimum:
        return "Minimum";
    case QSizePolicy::Fixed:
        return "Fixed";
    default:
        return "Unknown";
    }
}

QSizePolicy::Policy Form::policyByName(QString policyName)
{
    if (policyName == "Minimum") {
        return QSizePolicy::Minimum;
    }
    else if (policyName == "Expanding") {
        return QSizePolicy::Expanding;
    }
    else if (policyName == "Fixed") {
        return QSizePolicy::Fixed;
    }
    else
        return QSizePolicy::Preferred;
}

QWidget *Form::selectedWidget()
{
    return (ui->edtSelectWidget->currentIndex() == 0) ? ui->Widget1 : ui->Widget2;
}

void Form::on_edtSelectWidget_activated(int index)
{
    QWidget* widget = (index == 0) ? ui->Widget1 : ui->Widget2;
    readWidgetProps(widget);
}

void Form::on_edtSizePolicy_activated(int index)
{
    saveWidgetProps(selectedWidget());
}

void Form::on_edtStretchFactor_valueChanged(int arg1)
{
    saveWidgetProps(selectedWidget());
}

void Form::on_edtMinHeight_valueChanged(int arg1)
{
    saveWidgetProps(selectedWidget());
}

void Form::on_edtMaxHeight_valueChanged(int arg1)
{
    saveWidgetProps(selectedWidget());
}
